
clc

%%

clearvars -except SET

%%

model_name = 'regional' ;

experiment = SET.experiment ;

%%

hydata     = 2.52*4;                                % U.S. stock of housing to income
dhdata     = 0.29; 
dydata     = dhdata*hydata;

sigmac     = 1;                                     % CRRA coefficient  
nu         = 2;                                     % labor supply elasticity
etan       = 1;                                     % choice of units

hbar       = 1;  

r          = 0.02/4 ; 
gamma      = 0.985 ;
mbar       = (1-gamma)*dhdata ; 

omega      = 0.7 ;                                  % 
sigma      = 0.5 ;                                  % elasticity non-traded traded
kappa      = 4 ;                                    % tradable elasticity

rhohMA     = eps;
rhomMA     = eps;

switch experiment
    
    case 'baseline'

        alpha    = 3.44;

        gamma   = 0.985; 
        mbar    = (1-gamma)*dhdata; 

        rho     = 0.96;
        rhow    = 0.86;
        psi     = 21;

        rhoz       = 0.65 ;
        rhom       = 0.62 ;
        rhoh       = 0.87 ;
        rhon       = 0.59 ;
        rhob       = 0.80 ;

        sigmaz     = 0.55 ;
        sigmam     = 0.83 ;
        sigmah     = 0.71 ;
        sigman     = 2.20 ;
        sigmab     = 1.57 ;     
        
end

% Compute Steady State

euler     = @(xchat, dydata, alpha) 1 + dydata - xchat*(alpha/(alpha - 1) - 1/(alpha - 1)*xchat^(1-alpha))^(-1);

xchatss   = fzero(@(xchat) euler(xchat, dydata, alpha), 7);
Delta     = sigmac/(alpha - sigmac)*xchatss^(-alpha);
bet       = ((1 + r)*(1+Delta))^(-1);

rss       = r; 
qss       = 1/(1 + r - gamma); 
Fss       = (alpha/(alpha-1) - 1/(alpha-1)*xchatss.^(1-alpha));

muss      = (etan)^(sigmac/(sigmac + nu)).*Fss.^(nu*sigmac/(sigmac + nu))./(bet.*(1 + r)).^(nu/(sigmac + nu)); 
lambdass  =  muss.*(1 - bet*(1 + r))./(1 - bet*gamma); 
chatss    = (bet.*(1 + r).*muss).^(-1/sigmac); 
css       = (muss./etan).^(1/nu);
xss       = chatss.*xchatss; 
yss       = css;
nss       = yss;
wss       = etan*nss^nu/muss;

phss      = hydata.*css; 
etah      = phss./bet.*muss.*(1 - bet - lambdass./muss.*mbar);

qbss      = mbar*phss./(1 - gamma); 
qass      = (xss - css); 

pss       = phss/hbar;
bss       = qbss./qss; 
ass       = qass./qss;
dyss      = qss*bss/css;

yTss      = (1 - omega)*yss; 
yNss      = omega*yss;

params    = [bet; gamma; nu; etah; hbar; mbar; alpha; etan; sigmac; qss; omega; sigma; kappa; 1; 1; yss; rho; rhow; psi; log(wss); log(muss); log(nss); 0; 0; rhom; sigmam; rhoh; sigmah; rhoz; sigmaz; rhon; sigman; rhob; sigmab; bss; pss; rhomMA; rhohMA];

xsteady   = [pss; qss; muss; hbar; lambdass; xss; css; chatss; ass; bss; wss; nss; yss; yTss; yNss; 1; 1; 1];
xsteady   = log(xsteady); 

save input params xsteady

fprintf('\n')
fprintf('\n')
fprintf('Fraction Constrained:      Model                = %9.2f \n',       xchatss^(-alpha));
fprintf('Implied discount factor                         = %9.3f \n',       bet);
fprintf('Implied weight on housing, etah                 = %9.3f \n',       etah);
fprintf('Gap rho - r annualized, *100                    = %9.3f \n',       (1./bet - 1 - r)*4*100);
fprintf('\n')

fprintf('Equilibrium r:             Model, Data          = %9.2f %9.2f\n',  [r*4*100,             2.00]);
fprintf('Housing to Income:         Model, Data          = %9.2f %9.2f\n',  [pss*hbar/css,        hydata]);
fprintf('Debt to Housing:           Model, Data          = %9.2f %9.2f\n',  [qss*bss/(pss*hbar),  dhdata]);
fprintf('Debt to Income:            Model, Data          = %9.2f %9.2f\n',  [qss*bss/css,         dhdata*hydata]);
fprintf('\n')

%%

eval(['dynare ', model_name, '.mod noclearall nowarn']) ;

eval(['delete ', model_name, '.log']) ;
eval(['delete ', model_name, '_results.mat']) ;

dyn_in.M_  = M_ ;
dyn_in.oo_ = oo_ ;
dyn_in.solve = 1 ;
dyn_in.linearize_around_diff_y = 1 ;
dyn_in.dyn_linearize_point = oo_.steady_state ;
dyn_in.options_ = options_ ;

SET.M_ = dyn_in.M_ ;
SET.oo_ = dyn_in.oo_ ;
SET.options_ = options_ ;

out = dyn_to_str(dyn_in) ;

SET.mats.Q = out.mats.Q ; Q = SET.mats.Q ;
SET.mats.G = out.mats.G ; G = SET.mats.G ;

SET.str_mats.A0 = out.mats.A ;
SET.str_mats.A1 = out.mats.B ;
SET.str_mats.B0 = out.mats.D ;
SET.str_mats.D0 = out.mats.E ;
SET.str_mats.D2 = out.mats.D2 ;
SET.str_mats.Gamma = out.mats.Gamma ;

%% Extract variable details from Dynare output

n_ = M_.endo_nbr ; % Number of endogenous variables
l_ = M_.exo_nbr ;  % Number of exogenous variables
n_ = n_+1 ;        % Constant

SET.variable.n_ = n_ ; 
SET.variable.l_ = l_ ; 
SET.nparam      = M_.param_nbr ;

var_names   = cellstr(M_.endo_names) ;
param_names = cellstr(M_.param_names) ;
exo_var_names = cellstr(M_.exo_names) ;

for ii=1:n_-1
    eval(['SET.variable.',var_names{ii},'=',int2str(ii),';']) ;
end

for ii=1:l_
    eval(['SET.variable.shock.',exo_var_names{ii},'=',int2str(ii),';']) ;
end

for ii=1:M_.param_nbr
    eval(['SET.param_names.',param_names{ii},'=',int2str(ii),';']) ;
end

%% Compute steady-state

SET.y_ss = (eye(n_-1) - Q(1:n_-1,1:n_-1)) \ Q(1:n_-1,end)  ;
SET.y_ss = [SET.y_ss ; 1] ; % Constant

%% 

SET.r.Omega = eye(l_) ;

dyn_in.solve    = 0 ;